part of '../flutter_compute_tree.dart';

class Ref<T> {
  final _RefUnit _unit;
  T _value;

  Ref._(this._unit, this._value);

  T get value => _value;
  set value(T value) => write(value);

  void write(T value, {bool forceUpdate = false}) {
    if (forceUpdate || _value != value) {
      _value = value;
      _unit.notify();
    }
  }

  T update(T Function(T) fn, {bool force = false}) {
    final v = fn(_value);

    if (force || _value != v) {
      write(v, forceUpdate: force);
    }

    return v;
  }

  Trigger<Object> action(T Function(T value) fn) {
    final t = trigger();
    t.handler(() => update(fn), value);

    return t;
  }
}

class _RefUnit<T, K> extends Unit<Ref<T>, K> {
  late Ref<T> ref;
  final T Function() _factory;

  _RefUnit(this._factory);

  @override
  void init(CTNode node, K dep) {
    super.init(node, dep);
    ref = Ref._(this, _factory());
  }

  @override
  Unit<Ref<T>, K> update(CTNode node, K oldDep, K newDep, _) {
    super.update(node, oldDep, newDep, _);
    ref._value = _factory();
    return this;
  }

  @override
  Ref<T> unwrap() => ref;
}
