part of '../flutter_compute_tree.dart';

abstract class Trigger<T extends Object> {
  void call([T arg]);
  void Function() bind(T arg);

  Future<void> get called;

  Result<K> asyncHandler<K, R extends Object>(
    Future<K> Function() fn, {
    void Function()? abort,
  });

  K handler<K>(K Function() fn, K initialValue);
}

class _TriggerUnit<T extends Object> extends Unit<Trigger<T>, Object>
    implements Trigger<T> {
  final _completer = Completer<T>();

  var _dep = Object();

  @override
  Future<void> get called => _completer.future;

  _TriggerUnit();

  @override
  Trigger<T> unwrap() {
    return this;
  }

  @override
  void call([T? arg]) {
    arg ??= Object() as T;

    if (!_completer.isCompleted) {
      _completer.complete(arg);
    }

    _dep = Object();
    notify();
  }

  @override
  void Function() bind(T arg) {
    return () => call(arg);
  }

  @override
  Result<K> asyncHandler<K, R extends Object>(
    Future<K> Function() fn, {
    void Function()? abort,
  }) {
    return invokeAsync(
      fn,
      abort: abort,
      deferredUntil: called,
      dep: _dep,
    );
  }

  @override
  K handler<K>(K Function() fn, K initialValue) {
    return invoke.deferred<K, Object>(
      fn,
      until: called,
      initialValue: initialValue,
      dep: _dep,
    );
  }

  @override
  int get hashCode => _dep.hashCode;

  @override
  bool operator ==(Object other) {
    return other is _TriggerUnit<T> && other._dep == _dep;
  }
}
